SystemOrganization addCategory: #GitFS!
SystemOrganization addCategory: #'GitFS-Tests'!
SystemOrganization addCategory: #'GitFS-Exceptions'!

TestCase subclass: #GitFSTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GitFS-Tests'!

!GitFSTests class methodsFor: 'as yet unclassified' stamp: 'MaxLeske 5/11/2010 20:57'!
packageNamesUnderTest
	^#('GitFS')! !

!GitFSTests methodsFor: 'tests' stamp: 'MaxLeske 5/13/2010 14:00'!
testCreateRepository
	| path repository |
	path := '/alpha/testRepo.git'.
	repository := GitFS createRepositoryAt: path on: FSMemoryFilesystem new.
	self assert: repository exists.
	self assert: repository isInitialized! !

!GitFSTests methodsFor: 'tests' stamp: 'MaxLeske 5/12/2010 12:38'!
testDenyNew
	self should: [GitFS new] raise: Error description: '#new should disallow creation of new instances.'! !

!GitFSTests methodsFor: 'tests' stamp: 'MaxLeske 5/14/2010 14:18'!
testFindObjectByName
	| object foundObject repository |
	GitRepositoryCache instance clear.
	object := GitBlob bytes: 'testBlob'.
	repository := GitFS createRepositoryAt: 'testRepo' on: FSMemoryFilesystem new.
	repository storeObject: object.
	
	foundObject := GitFS findObjectNamed: object signature hexHash.
	self assert: foundObject = object! !

!GitFSTests methodsFor: 'tests' stamp: 'MaxLeske 5/14/2010 14:38'!
testGetObjectNamesForRepository
	| repository object1 object2 objectNames |
	GitRepositoryCache instance clear.
	object1 := GitBlob bytes: 'blob1'.
	object2 := GitBlob bytes: 'blob2'.
	repository := GitFS createRepositoryAt: 'testRepo.git' on: FSMemoryFilesystem new.
	repository 
		storeObject: object1;
		storeObject: object2.
	
	objectNames := GitFS getObjectNamesFor: repository.
	self assert: objectNames size = 2.
	self assert: (objectNames contains: [:each | each = (object1 signature)]).
	self assert: (objectNames contains: [:each | each = (object2 signature)])! !

!GitFSTests methodsFor: 'tests' stamp: 'MaxLeske 5/13/2010 14:26'!
testLoadingRepositoryNoDirectory
	| repoName memoryFS |
	GitRepositoryCache instance clear.
	memoryFS := FSMemoryFilesystem new.
	repoName := 'testRepo.git'.
	GitRepository on: (memoryFS root resolve: repoName).
	
	self 
		should: [GitFS loadRepositoryFrom: repoName on: memoryFS] 
		raise: RepositoryDirectoryDoesNotExist
		description: 'Something went terribly wrong!! The directory of the repository should not yet exist.'.! !

!GitFSTests methodsFor: 'tests' stamp: 'MaxLeske 5/13/2010 14:27'!
testLoadingRepositoryNotInitialized
	| repoName memoryFS |
	GitRepositoryCache instance clear.
	memoryFS := FSMemoryFilesystem new.
	repoName := 'testRepo.git'.
	memoryFS createDirectory: repoName.
	GitRepository on: (memoryFS root resolve: repoName).
	
	self 
		should: [GitFS loadRepositoryFrom: repoName on: memoryFS] 
		raise: RepositoryNotInitialized.! !

!GitFSTests methodsFor: 'tests' stamp: 'MaxLeske 5/13/2010 14:27'!
testOpenCachedRepositoryByName
	| repoName repository |
	GitRepositoryCache instance clear.
	repoName := 'testRepo.git'.
	repository := GitRepository on: (FSMemoryFilesystem new root resolve: repoName).
	self assert: repository = (GitFS openRepositoryNamed: repoName)! !

!GitFSTests methodsFor: 'tests' stamp: 'MaxLeske 5/12/2010 12:43'!
testRepositoryCacheAccess
	GitRepositoryCache instance clear.
	self assert: GitFS repositoryCache isEmpty.
	
 	GitRepositoryCache instance addRepository: (GitRepository on: FSMemoryFilesystem new working).
	self assert: GitFS repositoryCache isEmpty not description: 
		'The reference to the GitRepositoryCache object is not unique.'
		, 'Make sure there is only one instance of GitRepositoryCache.'.! !

Object subclass: #GitFS
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GitFS'!

!GitFS class methodsFor: 'operations' stamp: 'MaxLeske 5/14/2010 14:51'!
commit: anObject to: aGitRepository withMessage: aMessage
	| blob blobName commit commitName tree treeName |
	blob := GitBlob bytes: anObject.
	blobName := aGitRepository storeObject: blob.
	
	self error: 'the handling of the tree needs some work'.
	tree := GitTree entries: { GitTreeEntry 
							fileName: 'testFile' 
							mode: GitTreeEntry fileMode
							objectName: blobName }. "how can I reuse a tree? simply override existing trees?"
	
	
	commit := GitCommit new
		author: self withDefaultStamps;
		committer: self withDefaultStamps;
		tree: tree;
		message: aMessage;
		yourself.
	commitName := aGitRepository storeObject: commit.
	
	"do I need to make a tag here?"
	aGitRepository updateRef: 'HEAD' to: commitName! !

!GitFS class methodsFor: 'operations' stamp: 'MaxLeske 5/13/2010 15:35'!
createRepositoryAt: aString
	^self createRepositoryAt: aString on: FSDiskFilesystem current! !

!GitFS class methodsFor: 'operations' stamp: 'MaxLeske 5/13/2010 13:56'!
createRepositoryAt: aString on: anFSFilesystem
	| reference repository |
	reference := anFSFilesystem root resolve: aString.
	repository := GitRepository on: reference.
	repository gitInitialize.
	^repository! !

!GitFS class methodsFor: 'operations' stamp: 'MaxLeske 5/14/2010 14:14'!
findObjectNamed: sha1String
	^self repositoryCache objectAt: (GitObjectSignature from: sha1String)! !

!GitFS class methodsFor: 'operations' stamp: 'MaxLeske 5/14/2010 14:22'!
getObjectNamesFor: aGitRepository
	^aGitRepository looseObjectNames! !

!GitFS class methodsFor: 'operations' stamp: 'MaxLeske 5/12/2010 12:53'!
loadRepositoryFrom: aString
	^self loadRepositoryFrom: aString on: FSDiskFilesystem current! !

!GitFS class methodsFor: 'operations' stamp: 'MaxLeske 5/13/2010 15:47'!
loadRepositoryFrom: aString on: anFSFilesystem
	| reference repository |
	reference := anFSFilesystem root resolve: aString.
	repository := GitRepository on: reference.
	repository exists ifFalse: [RepositoryDirectoryDoesNotExist signal].
	repository isInitialized ifFalse: [^RepositoryNotInitialized signal].
	
	^repository! !

!GitFS class methodsFor: 'instance creation' stamp: 'MaxLeske 5/14/2010 18:36'!
new
	self error: 'Use the constructor methods to created instances.'! !

!GitFS class methodsFor: 'operations' stamp: 'MaxLeske 5/12/2010 12:42'!
openRepositoryNamed: aString
	^self repositoryCache repos detect: [:each | each reference basename = aString]! !

!GitFS class methodsFor: 'private' stamp: 'MaxLeske 5/12/2010 12:36'!
repositoryCache
	^GitRepositoryCache instance! !

Object subclass: #GitFSRepository
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GitFS'!

Error subclass: #RepositoryError
	instanceVariableNames: 'repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GitFS-Exceptions'!

RepositoryError subclass: #RepositoryDirectoryDoesNotExist
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GitFS-Exceptions'!

!RepositoryDirectoryDoesNotExist methodsFor: 'signaling' stamp: 'MaxLeske 5/12/2010 13:19'!
description
	self signal: 'A repository named ''' , repository reference basename
		, ''' does not exist. Did you create the repository correctly?'! !

!RepositoryError class methodsFor: 'instance creation' stamp: 'MaxLeske 5/12/2010 13:15'!
signalFor: aGitRepository
	self new signalFor: aGitRepository! !

!RepositoryError methodsFor: 'signaling' stamp: 'MaxLeske 5/12/2010 13:18'!
description
	self subclassResponsibility! !

!RepositoryError methodsFor: 'accessing' stamp: 'MaxLeske 5/14/2010 16:41'!
repository
	^repository! !

!RepositoryError methodsFor: 'signaling' stamp: 'MaxLeske 5/12/2010 13:21'!
signalFor: aGitRepository
	repository := aGitRepository.
	self signal: self description! !

RepositoryError subclass: #RepositoryNotInitialized
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GitFS-Exceptions'!

!RepositoryNotInitialized methodsFor: 'signaling' stamp: 'MaxLeske 5/12/2010 13:24'!
description
	self signal: 'The repository named ''' , repository reference basename
		, ''' has a directory but has not yet been initialized. I can''t open it.'! !
